//---------------------------------------------------------------------------

#include <vcl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Clipbrd.hpp"
#pragma hdrstop

#include "Sprites.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

void Sprites::NewSet()
{
  _sprite.clear();
}

// inserts a sprite after the position passed
void Sprites::InsertSprite(int n)
{
  int i;
  _SpriteI p;
  _Sprite s;

  p = _sprite.begin();
  for(i = 0; i <= n; i++)
    p++;
  _NewSprite(*_sprite.insert(p, s));
}

void Sprites::DeleteSprite(int n)
{
  int i;
  _SpriteI p;

  p = _sprite.begin();
  for(i = 0; i < n; i++)
    p++;
  _sprite.erase(p);
}

void Sprites::CopySprite(int n)
{
  unsigned int handle;
  HPALETTE APalette;
  unsigned short format;

  _sprite[n].data->SaveToClipboardFormat(format, handle, APalette);
  Clipboard()->SetAsHandle(format, handle);
}

void Sprites::PasteSprite(int n)
{
  TClipboard *pCB = Clipboard();

  if (pCB->HasFormat(CF_BITMAP))
    _sprite[n].data->LoadFromClipboardFormat(
                        CF_BITMAP, pCB->GetAsHandle(CF_BITMAP), 0);
}

// the bitmap passed must be 24 bit
void Sprites::RenderView(Graphics::TBitmap *b, int minWidth)
{
  char buf[10];
  int i, w, h;

  b->Height = 86;
  b->Width = _sprite.size() * 66;
  if (b->Width < minWidth)
    b->Width = minWidth;
  b->Canvas->Brush->Color = (TColor)0x808080;
  b->Canvas->FillRect(Rect(0, 0, b->Width, b->Height));
  for(i = 0; i < _sprite.size(); i++)
  {
    b->Canvas->Brush->Color = (TColor)0xA0A0A0;
    b->Canvas->FillRect(Rect((i * 66) + 1, 1, (i * 66) + 65, 65));
    w = _sprite[i].data->Width;
    if (w > 64)
      w = 64;
    h = _sprite[i].data->Height;
    if (h > 64)
      h = 64;
    DrawRect(Rect((i * 66) + 1, 1, (i * 66) + 1 + w, 1 + h), b->Canvas,
              Rect(0, 0, w, h), _sprite[i].data);
    sprintf(buf, "%i", i);
    w = (33 - (b->Canvas->TextWidth(buf) / 2));
    b->Canvas->Brush->Color = (TColor)0x808080;
    b->Canvas->TextOut((i * 66) + w, 68, buf);
  }
}

/*
  File format:

  int       : number of entries

  int       : animation timing (times per second, max of 60 fps)
  int       : number of animation frames
  char[32]  : name of the sprite
  int       : transparency color (-1 means no transparency)
  int       : width
  int       : height
  uchar[]   : bitmap data in RGB format (arranged [height][width])
*/
void Sprites::SaveToFile(char *fname)
{
  FILE *fp;
  int i, n;

  fp = fopen(fname, "wb");
  n = _sprite.size();
  fwrite(&n, 4, 1, fp);
  for(i = 0; i < _sprite.size(); i++)
  {
    fwrite(&_sprite[i].anim, 4, 1, fp);
    fwrite(&_sprite[i].frames, 4, 1, fp);
    fwrite(&_sprite[i].name, 32, 1, fp);
    if (_sprite[i].data->Transparent)
      n = _sprite[i].data->TransparentColor;
    else
      n = -1;
    fwrite(&n, 4, 1, fp);
    n = _sprite[i].data->Width;
    fwrite(&n, 4, 1, fp);
    n = _sprite[i].data->Height;
    fwrite(&n, 4, 1, fp);
    _sprite[i].data->PixelFormat = pf24bit;
    for(n = 0; n < _sprite[i].data->Height; n++)
      fwrite(_sprite[i].data->ScanLine[n], _sprite[i].data->Width, 3, fp);
  }
  fclose(fp);
}

void Sprites::MergeWithFile(char *fname)
{
  FILE *fp;
  int s, i, n;
  _Sprite spr;

  fp = fopen(fname, "rb");
  fread(&s, 4, 1, fp);
  s += _sprite.size();
  for(i = _sprite.size(); i < s; i++)
  {
    _sprite.push_back(spr);
    _NewSprite(_sprite[i]);
    fread(&_sprite[i].anim, 4, 1, fp);
    fread(&_sprite[i].frames, 4, 1, fp);
    fread(&_sprite[i].name, 32, 1, fp);
    _sprite[i].data = new Graphics::TBitmap();
    _sprite[i].data->PixelFormat = pf24bit;
    _sprite[i].data->Transparent = false;
    _sprite[i].data->TransparentMode = tmFixed;
    _sprite[i].data->TransparentColor = cl3DLight;
    fread(&n, 4, 1, fp);
    if (n != -1)
    {
      _sprite[i].data->Transparent = true;
      _sprite[i].data->TransparentColor = (TColor)n;
    }
    fread(&n, 4, 1, fp);
    _sprite[i].data->Width = n;
    fread(&n, 4, 1, fp);
    _sprite[i].data->Height = n;
    for(n = 0; n < _sprite[i].data->Height; n++)
      fread(_sprite[i].data->ScanLine[n], _sprite[i].data->Width, 3, fp);
  }
  fclose(fp);
}

void Sprites::ExportDefine(char *fname)
{
  FILE *fp;
  int i;

  fp = fopen(fname, "w");
  fprintf(fp, "/* Sprite header file generated by Sprite Studio */\n\n");
  for(i = 0; i < _sprite.size(); i++)
    if ((strlen(_sprite[i].name)) && (stricmp(_sprite[i].name, "SPR_")))
      fprintf(fp, "#define %s %i\n", _sprite[i].name, i);
  fclose(fp);
}

void Sprites::ExportEnum(char *fname)
{
  FILE *fp;
  char buf[80];
  int i, l;
  bool first;

  fp = fopen(fname, "w");
  fprintf(fp, "/* Sprite header file generated by Sprite Studio */\n\n");
  for(i = 0; i < _sprite.size(); i++)
    if ((strlen(_sprite[i].name)) && (stricmp(_sprite[i].name, "SPR_")))
      break;
  if (i < _sprite.size())
  {
    fprintf(fp, "enum SPR { ");
    l = 12;
    first = true;
    for(i = 0; i < _sprite.size(); i++)
      if ((strlen(_sprite[i].name)) && (stricmp(_sprite[i].name, "SPR_")))
      {
        sprintf(buf, "%s = %i", _sprite[i].name, i);
        if (!first)
        {
          l += 2;
          fprintf(fp, ", ");
        }
        else
          first = false;
        if (l + strlen(buf) > 72)
        {
          fprintf(fp, "\n           ");
          l = 12;
        }
        fprintf(fp, buf);
        l += strlen(buf);
      }
    fprintf(fp, " };\n");
  }
  fclose(fp);
}

void Sprites::SetTransC(int n, int c)
{
  _sprite[n].data->TransparentColor = (TColor)c;
}

void Sprites::_NewSprite(_Sprite &s)
{
  s.anim = 0;
  s.frames = 1;
  strcpy(s.name, "SPR_");
  s.data = new Graphics::TBitmap();
  s.data->PixelFormat = pf24bit;
  s.data->Transparent = false;
  s.data->TransparentMode = tmFixed;
  s.data->TransparentColor = cl3DLight;
}

