//---------------------------------------------------------------------------

#ifndef SpritesH
#define SpritesH
//---------------------------------------------------------------------------

#include <vector>
#include <string.h>

using namespace std;

class Sprites
{
  public:
    void NewSet();
    void InsertSprite(int n);
    void DeleteSprite(int n);
    void CopySprite(int n);
    void PasteSprite(int n);
    void RenderView(Graphics::TBitmap *b, int minWidth);
    void SaveToFile(char *fname);
    void MergeWithFile(char *fname);
    void ExportDefine(char *fname);
    void ExportEnum(char *fname);
    void SetTransC(int n, int c);

    int NumSprites()        { return _sprite.size(); }
    char *GetName(int n)    { return _sprite[n].name; }
    int GetWidth(int n)     { return _sprite[n].data->Width; }
    int GetHeight(int n)    { return _sprite[n].data->Height; }
    int GetAnim(int n)      { return _sprite[n].anim; }
    int GetFrames(int n)    { return _sprite[n].frames; }
    int GetTransC(int n)    { return _sprite[n].data->TransparentColor; }
    bool IsTrans(int n)     { return _sprite[n].data->Transparent; }
    void SetName(int n, char *c)  { strcpy(_sprite[n].name, c); }
    void SetAnim(int n, int a)    { _sprite[n].anim = a; }
    void SetFrames(int n, int f)  { _sprite[n].frames = f; }
    void SetTrans(int n, bool s)  { _sprite[n].data->Transparent = s; }
    Graphics::TBitmap *GetBitmap(int n) { return _sprite[n].data; }

  private:
    struct _Sprite
    {
      _Sprite() { data = NULL; }
      ~_Sprite() { delete data; }
      _Sprite(const _Sprite &r) { *this = r; }
      _Sprite & operator = (const _Sprite &r)
      {
        anim = r.anim;
        frames = r.frames;
        strcpy(name, r.name);
        data = new Graphics::TBitmap;
        data->Assign(r.data);
        return *this;
      }

      int anim,
          frames;
      char name[32];
      Graphics::TBitmap *data;
    };
    typedef vector<_Sprite>::iterator _SpriteI;

    void _NewSprite(_Sprite &s);

    vector<_Sprite> _sprite;
};

void DrawRect(const Windows::TRect &Dest, TCanvas *Canvas,
              const Windows::TRect &Source, Graphics::TBitmap *Bitmap)
{
  Graphics::TBitmap *b;

  b = new Graphics::TBitmap;
  b->Width = Dest.Width();
  b->Height = Dest.Height();
  b->Canvas->CopyRect(Rect(0, 0, b->Width, b->Height), Bitmap->Canvas, Source);
  b->TransparentColor = Bitmap->TransparentColor;
  b->Transparent = Bitmap->Transparent;
  Canvas->Draw(Dest.Left, Dest.Top, b);
  delete b;
}

#endif
