//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UMain.h"
#include "UAbout.h"
#include "Clipbrd.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMain *Main;
//---------------------------------------------------------------------------
__fastcall TMain::TMain(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMain::MainMenuExitClick(TObject *Sender)
{
  Main->Close();
}
//---------------------------------------------------------------------------
void __fastcall TMain::MainMenuAboutClick(TObject *Sender)
{
  About->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMain::MainMenuNewClick(TObject *Sender)
{
  if (!_CheckSave())
    return;

  _sprites.NewSet();
  _selected = -1;
  _filename = "";
  _modified = false;
  _animating = false;
  MainMenuClose->Enabled = false;
  _UpdateMenu();
  _UpdateSpriteList();
}
//---------------------------------------------------------------------------

void __fastcall TMain::FormCreate(TObject *Sender)
{
  Timer->Interval = 1000 / 60;
  _sprlist = new Graphics::TBitmap();
  _sprlist->PixelFormat = pf24bit;
  MainMenuNewClick(this);
}
//---------------------------------------------------------------------------

void __fastcall TMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
  if (!_CheckSave())
    CanClose = false;
}
//---------------------------------------------------------------------------

bool TMain::_CheckSave()
{
  if (_modified)
    switch(Application->MessageBox("You have made changes to the current "
            "sprite set.\n\nDo you want to save the changes?\n",
            "Sprite Studio", MB_YESNOCANCEL + MB_ICONWARNING))
    {
      case IDCANCEL:
        return false;
      case IDYES:
        if (_filename.Length())
          MainMenuSaveClick(this);
        else
        {
          if (!SaveDialog->Execute())
            return false;
          _filename = SaveDialog->FileName;
          _sprites.SaveToFile(_filename.c_str());
          _modified = false;
          _UpdateMenu();
        }
        break;
    }

  return true;
}

void __fastcall TMain::MainMenuOpenClick(TObject *Sender)
{
  if (!_CheckSave())
    return;

  OpenDialog->Title = "Open";
  if (!OpenDialog->Execute())
    return;

  _selected = -1;
  _filename = "";
  _animating = false;
  _modified = false;
  _sprites.NewSet();
  _sprites.MergeWithFile(OpenDialog->FileName.c_str());
  _filename = OpenDialog->FileName;
  _UpdateMenu();
  _UpdateSpriteList();
}
//---------------------------------------------------------------------------




void __fastcall TMain::SpriteListPaint(TObject *Sender)
{
  int p, w, s;

  SpriteList->Canvas->Brush->Color = clBlack;
  if (_sprites.NumSprites())
  {
    s = SpriteListBar->Position * 66;
    w = _sprites.NumSprites() * 66;
    if (_sprites.NumSprites() > 10)
      w = SpriteList->Width;
    SpriteList->Canvas->CopyRect(Rect(0, 0, w, SpriteList->Height),
                                  _sprlist->Canvas,
                                  Rect(s, 0, s + w, SpriteList->Height));
    p = (_selected - SpriteListBar->Position) * 66;
    SpriteList->Canvas->FrameRect(Rect(p, 0, p + 65, 66));
  }
}
//---------------------------------------------------------------------------




void __fastcall TMain::MainMenuInsertClick(TObject *Sender)
{
  _modified = true;
  if (_selected >= 0)
    _sprites.InsertSprite(_selected - 1);
  else
    _sprites.InsertSprite(_sprites.NumSprites() - 1);
  _UpdateSpriteList();
  if (_selected + SpriteListBar->Position > SpriteListBar->PageSize - 1)
    SpriteListBar->Position++;
}
//---------------------------------------------------------------------------

void TMain::_UpdateMenu()
{
  TClipboard *pCB = Clipboard();

  if (_selected == -1)
  {
    MainMenuCut->Enabled = false;
    MainMenuCopy->Enabled = false;
    MainMenuPaste->Enabled = false;
    MainMenuDelete->Enabled = false;
    SpriteInfoEditName->Enabled = false;
    SpriteInfoEditAnim->Enabled = false;
    SpriteInfoEditFrames->Enabled = false;
    SpriteInfoEditTrans->Enabled = false;
    SpriteInfoEditAnimUD->Enabled = false;
    SpriteInfoEditFramesUD->Enabled = false;
    SpriteInfoEditName->Text = "SPR_";
    SpriteInfoEditWidth->Text = "0";
    SpriteInfoEditHeight->Text = "0";
    SpriteInfoEditAnim->Text = "0";
    SpriteInfoEditFrames->Text = "1";
    SpriteInfoEditTrans->Checked = false;
    _transcolor = cl3DLight;
    ButtonAnimate->Enabled = false;
    SpriteInfoEditUpdate->Enabled = false;
  }
  else
  {
    MainMenuCut->Enabled = true;
    MainMenuCopy->Enabled = true;
    MainMenuDelete->Enabled = true;
    SpriteInfoEditName->Enabled = true;
    SpriteInfoEditAnim->Enabled = true;
    SpriteInfoEditFrames->Enabled = true;
    SpriteInfoEditTrans->Enabled = true;
    SpriteInfoEditAnimUD->Enabled = true;
    SpriteInfoEditFramesUD->Enabled = true;
    SpriteInfoEditName->Text = AnsiString(_sprites.GetName(_selected));
    SpriteInfoEditWidth->Text = AnsiString(_sprites.GetWidth(_selected));
    SpriteInfoEditHeight->Text = AnsiString(_sprites.GetHeight(_selected));
    SpriteInfoEditAnim->Text = AnsiString(_sprites.GetAnim(_selected));
    SpriteInfoEditFrames->Text = AnsiString(_sprites.GetFrames(_selected));
    SpriteInfoEditTrans->Checked = _sprites.IsTrans(_selected);
    _transcolor = _sprites.GetTransC(_selected);
    ButtonAnimate->Enabled = true;
    SpriteInfoEditUpdate->Enabled = true;
  }
  if (pCB->HasFormat(CF_BITMAP))
    MainMenuPaste->Enabled = true;
  MainMenuSave->Enabled = false;
  MainMenuSaveAs->Enabled = false;
  if (_modified)
  {
    if (_filename.Length())
      MainMenuSave->Enabled = true;
    MainMenuSaveAs->Enabled = true;
    MainMenuClose->Enabled = true;
    StatusBar->SimpleText = "Modified";
  }
  else
    StatusBar->SimpleText = "";
  if (_filename.Length())
  {
    Main->Caption = "Sprite Studio - " +
                  _filename.SubString(_filename.LastDelimiter("\\") + 1, 250);
    MainMenuClose->Enabled = true;
  }
  else
    Main->Caption = "Sprite Studio";
  SpriteViewPaint(this);
  SpriteInfoEditTransColorPaint(this);
}

void TMain::_UpdateSpriteList()
{
  _sprites.RenderView(_sprlist, SpriteList->Width);
  if (_sprites.NumSprites() < SpriteListBar->PageSize)
    SpriteListBar->Max = 0;
  else
    SpriteListBar->Max = _sprites.NumSprites() - SpriteListBar->PageSize;
  SpriteList->Repaint();
}

void __fastcall TMain::SpriteListBarChange(TObject *Sender)
{
  _UpdateSpriteList();
}
//---------------------------------------------------------------------------



void __fastcall TMain::SpriteListMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  if ((X / 66) > _sprites.NumSprites() - 1)
    return;

  _selected = SpriteListBar->Position + (X / 66);
  _animating = false;
  ButtonAnimate->Caption = "Animate";
  _UpdateMenu();
  _UpdateSpriteList();
}
//---------------------------------------------------------------------------


void __fastcall TMain::MainMenuDeleteClick(TObject *Sender)
{
  _modified = true;
  _sprites.DeleteSprite(_selected);
  if (_sprites.NumSprites() < _selected + 1)
    _selected--;
  _UpdateMenu();
  _UpdateSpriteList();
}
//---------------------------------------------------------------------------

void __fastcall TMain::MainMenuSaveAsClick(TObject *Sender)
{
  if (!SaveDialog->Execute())
    return;

  _filename = SaveDialog->FileName;
  _sprites.SaveToFile(_filename.c_str());
  _modified = false;
  _UpdateMenu();
}
//---------------------------------------------------------------------------


void __fastcall TMain::MainMenuCopyClick(TObject *Sender)
{
  _sprites.CopySprite(_selected);
}
//---------------------------------------------------------------------------

void __fastcall TMain::MainMenuCutClick(TObject *Sender)
{
  MainMenuCopyClick(this);
  MainMenuDeleteClick(this);
}
//---------------------------------------------------------------------------


void __fastcall TMain::MainMenuPasteClick(TObject *Sender)
{
  TClipboard *pCB = Clipboard();

  if (!pCB->HasFormat(CF_BITMAP))
    return;

  if (_selected >= 0)
    _sprites.PasteSprite(_selected);
  else
  {
    MainMenuInsertClick(this);
    _sprites.PasteSprite(_sprites.NumSprites() - 1);
  }
  _UpdateSpriteList();
}
//---------------------------------------------------------------------------

void __fastcall TMain::FormClick(TObject *Sender)
{
  _selected = -1;
  _animating = false;
  ButtonAnimate->Caption = "Animate";
  _UpdateMenu();
  _UpdateSpriteList();
}
//---------------------------------------------------------------------------


void __fastcall TMain::MainMenuEditClick(TObject *Sender)
{
  _UpdateMenu();  
}
//---------------------------------------------------------------------------



void __fastcall TMain::SpriteInfoEditUpdateClick(TObject *Sender)
{
  int i;

  i = atoi(SpriteInfoEditAnim->Text.c_str());
  if (i < 0)
    i = 0;
  else if (i > 60)
    i = 60;
  _sprites.SetAnim(_selected, i);
  i = atoi(SpriteInfoEditFrames->Text.c_str());
  if (i < 1)
    i = 1;
  _sprites.SetFrames(_selected, i);
  _sprites.SetTrans(_selected, SpriteInfoEditTrans->Checked);
  _sprites.SetTransC(_selected, _transcolor);
  _sprites.SetName(_selected, SpriteInfoEditName->Text.c_str());
  _UpdateMenu();
  _UpdateSpriteList();
  _modified = true;
}
//---------------------------------------------------------------------------


void __fastcall TMain::SpriteViewPaint(TObject *Sender)
{
  int n, i,
      sw, sh, dw, dh;
  Graphics::TBitmap *b;

  if (_selected == -1)
  {
    SpriteView->Canvas->Brush->Color = cl3DLight;
    SpriteView->Canvas->FillRect(
                          Rect(0, 0, SpriteView->Width, SpriteView->Height));
    return;
  }
  b = new Graphics::TBitmap;
  n = 0;
  if (_animating)
  {
    _animate++;
    i = _sprites.GetAnim(_selected);
    n = ((int)(_animate / (60.0 / i))) % _sprites.GetFrames(_selected);
  }
  n += _selected;
  if ((n >= 0) && (n < _sprites.NumSprites()))
  {
    sw = _sprites.GetBitmap(n)->Width;
    sh = _sprites.GetBitmap(n)->Height;
    if ((sw) && (sh))
    {
      if (MainMenuScaled->Checked)
      {
        if (sw > sh)
        {
          dw = SpriteView->Width;
          dh = ((float)sh / sw) * SpriteView->Height;
        }
        else
        {
          dw = ((float)sw / sh) * SpriteView->Width;
          dh = SpriteView->Height;
        }
      }
      else
      {
        if (sw > SpriteView->Width)
          sw = SpriteView->Width;
        if (sh > SpriteView->Height)
          sh = SpriteView->Height;
        dw = sw;
        dh = sh;
      }
      b->Canvas->Brush->Color = cl3DLight;
      b->Width = dw;
      b->Height = dh;
      b->Canvas->FillRect(Rect(0, 0, b->Width, b->Height));
      DrawRect(Rect(0, 0, dw, dh), b->Canvas,
                Rect(0, 0, sw, sh), _sprites.GetBitmap(n));
      SpriteView->Canvas->Draw(0, 0, b);
    }
  }
  delete b;
}
//---------------------------------------------------------------------------


void __fastcall TMain::ButtonAnimateClick(TObject *Sender)
{
  if (!_animating)
  {
    if (_selected + _sprites.GetFrames(_selected) > _sprites.NumSprites())
    {
      Application->MessageBox("Cannot animate: the number of animation frames"
                              " exceeds the remaining number of sprites!",
                              "Sprite Studio", MB_OK + MB_ICONWARNING);
      return;
    }
    if (!_sprites.GetAnim(_selected))
    {
      Application->MessageBox("Cannot animate: the animation timing is zero!",
                              "Sprite Studio", MB_OK + MB_ICONWARNING);
      return;
    }
    ButtonAnimate->Caption = "Stop";
    _animate = 0;
  }
  else
    ButtonAnimate->Caption = "Animate";

  _animating = !_animating;
}
//---------------------------------------------------------------------------

void __fastcall TMain::TimerTimer(TObject *Sender)
{
  if (_animating)
    SpriteViewPaint(this);
}
//---------------------------------------------------------------------------




void __fastcall TMain::MainMenuScaledClick(TObject *Sender)
{
  MainMenuScaled->Checked = true;
  SpriteView->Canvas->Brush->Color = cl3DLight;
  SpriteView->Canvas->FillRect(Rect(0, 0,
                                  SpriteView->Width, SpriteView->Height));
  _UpdateMenu();
}
//---------------------------------------------------------------------------

void __fastcall TMain::MainMenuNormalClick(TObject *Sender)
{
  MainMenuNormal->Checked = true;
  SpriteView->Canvas->Brush->Color = cl3DLight;
  SpriteView->Canvas->FillRect(Rect(0, 0,
                                  SpriteView->Width, SpriteView->Height));
  _UpdateMenu();
}
//---------------------------------------------------------------------------


void __fastcall TMain::SpriteInfoEditTransColorPaint(TObject *Sender)
{
  if (_selected != -1)
    SpriteInfoEditTransColor->Canvas->Brush->Color = (TColor)_transcolor;
  else
    SpriteInfoEditTransColor->Canvas->Brush->Color = cl3DLight;
  SpriteInfoEditTransColor->Canvas->FillRect(Rect(0, 0,
                    SpriteInfoEditTrans->Width, SpriteInfoEditTrans->Height));
}
//---------------------------------------------------------------------------


void __fastcall TMain::SpriteViewMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  if (_selected != -1)
    _transcolor = SpriteView->Canvas->Pixels[X][Y];
  SpriteInfoEditTransColorPaint(this);
}
//---------------------------------------------------------------------------

void __fastcall TMain::MainMenuSaveClick(TObject *Sender)
{
  _sprites.SaveToFile(_filename.c_str());
  _modified = false;
}
//---------------------------------------------------------------------------


void __fastcall TMain::MainMenuFileClick(TObject *Sender)
{
  _UpdateMenu();
}
//---------------------------------------------------------------------------

void __fastcall TMain::MainMenuMergeClick(TObject *Sender)
{
  OpenDialog->Title = "Merge";
  if (!OpenDialog->Execute())
    return;

  _animating = false;
  _sprites.MergeWithFile(OpenDialog->FileName.c_str());
  _UpdateMenu();
  _UpdateSpriteList();
  _modified = true;
}
//---------------------------------------------------------------------------

void __fastcall TMain::MainMenuCloseClick(TObject *Sender)
{
  MainMenuNewClick(this);
}
//---------------------------------------------------------------------------

void __fastcall TMain::FormDestroy(TObject *Sender)
{
  delete _sprlist;
}
//---------------------------------------------------------------------------


void __fastcall TMain::ExportMenuDefineClick(TObject *Sender)
{
  if (!ExportDialog->Execute())
    return;

  _sprites.ExportDefine(ExportDialog->FileName.c_str());
}
//---------------------------------------------------------------------------

void __fastcall TMain::ExportMenuEnumClick(TObject *Sender)
{
  if (!ExportDialog->Execute())
    return;

  _sprites.ExportEnum(ExportDialog->FileName.c_str());
}
//---------------------------------------------------------------------------

